import { EventAction, getFont, Pen } from "@meta2d/core";
import { MonitoringPointData } from "@/types";

const enum MonitorPointStatus {
  /**
   * 离线
   */
  Offline = -1,
  /**
   * 关闭
   */
  Closed = 0,
  /**
   * 正常
   */
  Normal = 1,
  /**
   * 预警
   */
  Warning = 2,
  /**
   * 超标
   */
  Exceed = 3,
}

const isPointNormal = (point: MonitoringPointData) => {
  const { status } = point;
  return status !== MonitorPointStatus.Exceed;
};

export interface MonitorPointPen extends Pen {
  monitorPointInfo: MonitoringPointData;
  monitorPointHighlighted?: boolean;
}

export function monitorPoint(
  ctx: CanvasRenderingContext2D,
  pen: MonitorPointPen
): void {
  const { x, y, width, height } = pen.calculative.worldRect;
  const { name, temperature, humidity } = pen.monitorPointInfo;
  const isLock = meta2d.data().locked;
  const isNormal = isPointNormal(pen.monitorPointInfo);
  ctx.save();

  // 状态图标中心坐标
  const iconX = x + width / 2;
  const iconY = y + height / 2;
  const iconColor = !isLock || isNormal ? "103, 194, 58" : "245, 108, 108"; // 正常状态 - 绿色、异常状态 - 红色

  // 绘制三个同心圆
  // 选中监测点时绘制背景圆，用于高亮
  if (pen.monitorPointHighlighted) {
    ctx.beginPath();
    ctx.arc(iconX, iconY, 40, 0, Math.PI * 2);
    ctx.fillStyle = `rgba(${iconColor}, 0.3)`;
    ctx.fill();
  }

  // 最大圆（直径24）
  ctx.beginPath();
  ctx.arc(iconX, iconY, 12, 0, Math.PI * 2);
  ctx.fillStyle = `rgba(${iconColor}, 0.3)`;
  ctx.fill();

  // 中圆（直径12）
  ctx.beginPath();
  ctx.arc(iconX, iconY, 6, 0, Math.PI * 2);
  ctx.fillStyle = `rgba(${iconColor}, 0.6)`;
  ctx.fill();

  // 最小圆（直径6）
  ctx.beginPath();
  ctx.arc(iconX, iconY, 3, 0, Math.PI * 2);
  ctx.fillStyle = `rgba(${iconColor}, 1)`;
  ctx.fill();

  ctx.font = getFont({
    fontWeight: "500",
    fontSize: 12,
    lineHeight: 22,
  });

  ctx.fillStyle = "rgba(0, 0, 0, 0.65)";
  ctx.textBaseline = "middle";

  // 绘制右侧文字
  if (!isLock) {
    ctx.fillText(name, iconX + 25, iconY);
  }

  // 绘制下方文字
  if (isLock && !isNormal) {
    ctx.textAlign = "center";
    ctx.fillText(`${temperature}℃｜${humidity}%`, iconX, iconY + 35);
  }

  ctx.restore();
}

export const monitorPointData: Pen = {
  name: "monitorPoint",
  width: 214,
  height: 56,
  events: [
    {
      name: "click",
      action: EventAction.Emit,
      value: "highlightMonitorPoint",
    },
  ],
};
